package com.gnv.eneg;

import java.io.IOException;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.Resource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.method.annotation.MvcUriComponentsBuilder;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.gnv.eneg.storage.StorageFileNotFoundException;
import com.gnv.eneg.storage.StorageService;

import com.gnv.eneg.PairAPI;
import java.util.concurrent.CountDownLatch;

/**
 * Ariane 11/12/2017
 * From: https://spring.io/guides/gs/uploading-files/
 */

@Controller
public class GNV {

    private final StorageService storageService;

    // Where the bedPairFile is stored.
    public String bedPairFilePath;
    private String directory = "/home/ariane/Documents/bodenlab/ASR/gnvweb/eneg/upload-dir/";

    // The PairApi which stores the current instance of the user.
    PairAPI pairAPI;


    @Autowired
    public GNV(StorageService storageService) {
        this.storageService = storageService;
    }

    @GetMapping("/")
    public String listUploadedFiles(Model model) throws IOException {

        model.addAttribute("files", storageService.loadAll().map(
                path -> MvcUriComponentsBuilder.fromMethodName(GNV.class,
                        "serveFile", path.getFileName().toString()).build().toString())
                .collect(Collectors.toList()));

        // Set the networks to be None until a user has uploaded a file.
        model.addAttribute("networksInBedFile", "None");

        return "index";
    }

    @GetMapping("/files/{filename:.+}")
    @ResponseBody
    public ResponseEntity<Resource> serveFile(@PathVariable String filename) {

        Resource file = storageService.loadAsResource(filename);

        return ResponseEntity.ok().header(HttpHeaders.CONTENT_DISPOSITION,
                "attachment; filename=\"" + file.getFilename() + "\"").body(file);
    }

    @PostMapping("/")
    public String handleFileUpload(@RequestParam("file") MultipartFile file,
            RedirectAttributes redirectAttributes) {

        storageService.store(file);

        /**
         *  Gets the file and starts the process of getting the networks from the file.
         *
         *  Initially used to get all the networks within a bedPairFile.
         *
         *  These are relative to the chromosone.
         */

        bedPairFilePath = directory + file.getOriginalFilename();
        pairAPI = new PairAPI(bedPairFilePath);

        // Set the results flag to be true to show the results page.
        redirectAttributes.addFlashAttribute("results", true);

        redirectAttributes.addFlashAttribute("message",
                "You successfully uploaded " + file.getOriginalFilename() + "!");

        return "redirect:/";
    }


    /**
     * An Ajax call performed when the user uploads the BED file.
     * Only returns once the file has fully loaded.
     * @param request
     * @param model
     * @return
     */

    @RequestMapping(value = "/", method = RequestMethod.GET, params = {"request"})
    public @ResponseBody String getNetworksInUploadedBedFile(@RequestParam("request") String request, Model model) {

        return pairAPI.getNetworksInUploadedBedFile().toString();
    }


    /**
     * An Ajax call performed when the user clicks on a network.
     *
     * Returns the interactions involved in that network.
     *
     * @param request
     * @param model
     * @return
     * @throws IOException
     */
    @RequestMapping(value = "/", method = RequestMethod.GET, params = {"chrom", "start", "end"})
    public @ResponseBody String getNetwork(@RequestParam("chrom") String chrom, @RequestParam("start") String start,  @RequestParam("end") String end, Model model) {

        return pairAPI.getNetwork(chrom, Integer.parseInt(start), Integer.parseInt(end)).toString();
    }


    @ExceptionHandler(StorageFileNotFoundException.class)
    public ResponseEntity<?> handleStorageFileNotFound(StorageFileNotFoundException exc) {
        return ResponseEntity.notFound().build();
    }

}
