package com.gnv.eneg;

import pairs.*;
import json.*;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.*;


/**
 * BINFKIT API for using the BED pair calls from Alex's code.
 *
 * Created by ariane on 8/11/17.
 */
public class PairAPI {

    private static BedPairFile bpf;

    // Passed to the class, it is the users' uploaded bed file.
    private static String bedPairFilePath;

    // Set variables. Default values but updateable.
    private static int maxPairSize = 2000000;
    private static int padding = 0;

    // Where data i.e. JSON is stored locally
    private static String dataPath = "/home/ariane/Documents/bodenlab/ASR/gnvweb/eneg/src/main/resources/public/data/vis/";


    public PairAPI(String bedPairFilePath) {
        this.bpf = new BedPairFile(bedPairFilePath);
        this.bedPairFilePath = bedPairFilePath;
    }


    /**
     * Gets the network (or networks) within a specific region within a chromosone.
     *
     * Returns the Network as a JSON array or elements.
     */
    public static JSONArray getNetwork(String chrom, int start, int end) {
        bpf.getNetworkAPI(bedPairFilePath, maxPairSize, padding, chrom, start, end);
        JSONArray network = bpf.jsonNetwork;
        return network;
    }


    /**
     * Gets the networks within a users uploaded bed file.
     *
     * Returns these as a JSON array of the start, end and chromosone of the network.
     *
     */
    public static JSONArray getNetworksInUploadedBedFile() {
        bpf.getNetworksAsJSONAccrossBedFile(bedPairFilePath, maxPairSize, padding);
        JSONArray networks = bpf.jsonNetworksInBedFile;
        return networks;
    }
}