/**
 * Processes the data using crossfilter.
 * help:
 * top sorts descending (text or num) on the field
 * returns the top x amount. Can sort in terms of top infinity --> to get a sorted list.
 * bottom sorts ascending (but same same).
 *
 * to remove dimesnion = dispose
 * Can have max 32 (> 8 dimensions cause it to be very slow).
 * Can make an array to have several elements in a dimensions (i.e. start and end).
 * .all() on the grouped data exposes the data to be able to use it in Javascrippt
 *
 * filterRangd([>=min, <max]); Can also do on strings
 * there is also a filter for a function.
 */


/**
 * Helps with debugging. Native cross filter objects can't be viewed properly in
 * console.log.
 */
function get_filter(filter) {
    var f = eval(filter);
    if (typeof (f.length) != "undefined") {
    } else {
    }
    if (typeof (f.top) != "undefined") {
        f = f.top(Infinity);
    } else {
    }
    if (typeof (f.dimension) != "undefined") {
        f = f.dimension(function (d) {
            return "";
        }).top(Infinity);
    } else {
    }
    return f;
    //console.log(filter + "(" + f.length + ") = " + JSON.stringify(f).replace("[", "[\n\t").replace(/}\,/g, "},\n\t").replace("]", "\n]"));
}

/**
 * ----------------------------------------------------------------------------
 *                      Setup the data
 * ----------------------------------------------------------------------------
 */

var processed_data = {
    curr_chrom: 'chr1', // keep track of the current chromosone.
    curr_gene: 'gene',
};


var set_up_genome_data = function (data) {
    // Set up the crossfilter facts. --> use a JSON object i.e. data
    // that contains elements (array) these get translated to facts.
    // Data contains the following:
    //      1.chrom     2.start     3.end   4.gene or network
    var genome_facts = crossfilter(data);

    // Store globally
    processed_data["genome_facts"] = genome_facts;
    processed_data["fact_size"] = genome_facts.size();
}

var add_network_data = function(network_data) {
    // Add the network data to the genome data
    processed_data.genome_facts.add(network_data);

}

/**
 * ----------------------------------------------------------------------------
 *                      Create dimensions
 * ----------------------------------------------------------------------------
 */
var create_type_dimension = function () {
    // Create a dimension that allows easy filtering on the networks or gene
    var type_dimension = processed_data.genome_facts.dimension(function (d) {
        return d.type;
    });

    // Store globally
    processed_data["type_dimension"] = type_dimension;

}


var create_start_dimension = function () {
    // Dimensions are stateful.
    // Create a dimesnsion on the start of either the gene or network
    var start_dimension = processed_data.genome_facts.dimension(function (d) {
        return d.start;
    });

    // Store globally
    processed_data["start_dimension"] = start_dimension;

}

var create_end_dimension = function () {
    // Dimensions are stateful.
    // Create a dimesnsion on the end of either the gene or network
    var end_dimension = processed_data.genome_facts.dimension(function (d) {
        return d.end;
    });

    // Store Globally
    processed_data["end_dimension"] = end_dimension;

}

var create_chrom_dimension = function () {
    // Create a dimesnsion on the chromosone of the gene or network
    var chrom_dimension = processed_data.genome_facts.dimension(function (d) {
        return d.chrom;
    });

    // Store Globally
    processed_data["chrom_dimension"] = chrom_dimension;

}

/**
 * ----------------------------------------------------------------------------
 *                      Create groups
 * ----------------------------------------------------------------------------
 */
var create_chrom_group = function() {
    // Want to get all the chromosones as a group

    processed_data["chrom_group"] = processed_data.chrom_dimension.group(function(d) {return d.split('_');}).all();

};

/**
 * ----------------------------------------------------------------------------
 *        Sort functions called during vis interaction
 * ----------------------------------------------------------------------------
 */

var clear_filters = function() {
    processed_data.start_dimension.filterAll();
    processed_data.end_dimension.filterAll();
    processed_data.chrom_dimension.filterAll();
    processed_data.type_dimension.filterAll();
}

var filter_on_position = function(start, end) {
    // Clear the associated filters.
    processed_data.start_dimension.filterAll();
    processed_data.end_dimension.filterAll();

    // Filters the data based on the start and end positions
    // filterRange([>=min, <max]); Can also do on strings
    processed_data.start_dimension.filterRange([start, end]);
    processed_data.end_dimension.filterRange([start, end]);
    // Now the underlying data has changed (will need to redraw)
    return processed_data.end_dimension.top(Infinity);
}

var filter_on_chrom = function(chrom) {
    // Clear associated filter
    processed_data.chrom_dimension.filterAll();

    // Use an exact filter as we only want that specific chromosone.
    processed_data.chrom_dimension.filterExact(chrom);

    // Now the underlying data has changed (will need to redraw)
    return get_filter(processed_data.genome_facts);
}



var filter_on_networks = function() {
    // Clear associated filter
    processed_data.type_dimension.filterAll();

    // Allows us to only look at networks (the genes will have the gene ID in
    // that collumn).
    processed_data.type_dimension.filterExact("network");
    // Now the underlying data has changed (will need to redraw)
    return get_filter(processed_data.genome_facts);
}



var filter_on_gene = function(gene) {
    // Clear associated filter.
    processed_data.type_dimension.filterAll();

    // Here we filter using the type dimension as that will either have "network"
    // or the gene ID.
    processed_data.type_dimension.filterExact(gene);
    // Now the underlying data has changed (will need to redraw)
    return get_filter(processed_data.genome_facts);
}










/**
 * ====================================================================================================================
 *                              Create a new crossfilter instance for each network range
 * ====================================================================================================================
 */

var set_up_network_data = function (data) {
     // Set up the crossfilter facts. --> use a JSON object i.e. data
     // that contains elements (array) these get translated to facts.
     // Data contains the following:
     //      1.start     2.end   3. Array of networks that have that start and end.
     var network_facts = crossfilter(data);

     // Store globally
     processed_data["network_facts"] = network_facts;
}


// Need to remove the facts when we get a new network
var remove_network_data = function() {
    if (processed_data.network_facts != []) {
        processed_data.network_facts.remove();
    }
}

/**
 * ============================================================================
 *                        Create interaction dimensions
 * ============================================================================
 */

/**
 * Make a dimension on the network ID to make it quick to filter on that ID.
 */
var create_network_dimension = function() {
    // Create a dimesnsion on the chromosone of the gene or network
    var network_dimension = processed_data.network_facts.dimension(function (d) {
        return d.network_id;
    });

    // Store Globally
    processed_data["network_dimension"] = network_dimension;
};

/**
 * Make a dimension tpo easily filter between edges and interactions
 */
var create_network_type_dimension = function() {
    // Create a dimesnsion on the chromosone of the gene or network
    var network_type_dimension = processed_data.network_facts.dimension(function (d) {
        return d.type;
    });

    // Store Globally
    processed_data["network_type_dimension"] = network_type_dimension;
};


/**
 * Make a dimension on the start to make it easy to filter.
 */
var create_interaction_start_dimension = function() {
    // Create a dimesnsion on the chromosone of the gene or network
    var interaction_start_dimension = processed_data.network_facts.dimension(function (d) {
        return d.start;
    });

    // Store Globally
    processed_data["interaction_start_dimension"] = interaction_start_dimension;
};


/**
 * Make a dimension on the start to make it easy to filter.
 */
var create_interaction_end_dimension = function() {
    // Create a dimesnsion on the chromosone of the gene or network
    var interaction_end_dimension = processed_data.network_facts.dimension(function (d) {
        return d.end;
    });

    // Store Globally
    processed_data["interaction_end_dimension"] = interaction_end_dimension;
}

/**
 *  ===================================================================================================================
 *                           Want filters: 1. Networks (need to check if in array)
 *  ===================================================================================================================
 */

var remove_network_filters = function() {
    processed_data.network_dimension.filterAll();
    processed_data.network_type_dimension.filterAll();
    processed_data.interaction_start_dimension.filterAll();
    processed_data.interaction_end_dimension.filterAll();
}

 var filter_on_network = function(network_id) {
    processed_data.network_dimension.filterAll();

    // want the interactions with that network
    processed_data.network_dimension.filterExact(network_id);

    return processed_data.network_type_dimension.top(Infinity);
 }

// Either returns all interactions or all egdes
var filter_on_network_type = function(type) {
    processed_data.network_type_dimension.filterAll();

    // Only want interactions not edges
    type_filter = processed_data.network_type_dimension.filterExact(type);

    return processed_data.network_dimension.top(Infinity);
 }

 var filter_interactions_on_position = function(start, end) {
    // Clear the associated filters.
    processed_data.interaction_start_dimension.filterAll();
    processed_data.interaction_end_dimension.filterAll();

    // Filters the data based on the start and end positions
    // filterRange([>=min, <max]); Can also do on strings
    processed_data.interaction_start_dimension.filterRange([start, end]);
    processed_data.interaction_end_dimension.filterRange([start, end]);

    // Now the underlying data has changed (will need to redraw)
    return get_filter(processed_data.network_facts);
 }