/**
 * The main file that renders a visualisation of circuits in DNA.
 */
var data = {
    nodes: [],
    edges: [],
    min_position: 1111111111111111111,
    max_position: 0,
    max_dist: 0,
    min_dist: 111111111111111111111111111111111,
};

var threeD_attr = set_up_3D_attr();

var dna_attr = {
    circle: {
        radius: 60, // max is radius + min
        min_radius: 20,
    },
    rect: {
        node_points: [],
        colour_outer: "black",
        colour_inner: "green",
        width: 6,
        height: 5,
    },
    semi_circle: {
        outer_radius: 100, // height, width = 2 * radius
        inner_radius: 90,
        stroke_colour: "white",
        detailed_radi: {},
        rad_types: "normal", // is either normal for an even spread or multi for
        // radi dependent on num edges
    },
    main_strand: {
        stroke_width: 1,
        stroke_colour: "black", //"#627071", // nearly black
        tail_length: 50
    },
    dna_strand: {
        stroke_width: 1,
        internal_dist: 50,
        external_dist: 200, // used to set a limit for how tall the strand is
        dist_stroke_colour: "#627071"
    },
    vis: {
        scale: {},
        width: 1200,
        height: 400,
        fdg_rad: 5,
    }
};
/**
 * When a user clicks on a network there may be multiple networks involved.
 *
 * Want to add each of the interactions as points on the page and then
 * create  networks from the combinations.
 */
var process_networks = function () {
    var network_data = dna_attr.data;
    // Keep an array of all the facts
    var facts = new Array();
    dna_attr.num_networks = network_data.length;

    for (var network in network_data) {
        for (var n in network_data[network]['nodes'][0]) {
            var node = network_data[network]['nodes'][0][n];
            node.type = 'interaction';
            node.chrom = node.chrom[0];
            node.start = +node.start[0];
            node.end = +node.end[0];
            node.id = node.label[0];
            node.label = node.label[0];
            node.num_edges = +node.num_edges[0];
            node.network_id = +network;
            facts.push(node);
        }
        for (var e in network_data[network]['edges'][0]) {
            var edge = network_data[network]['edges'][0][e];
            edge.type = 'edge';
            edge.target = edge.to[0];
            edge.source = edge.from[0];
            edge.to = edge.to[0];
            edge.from = edge.from[0];
            edge.label = edge.label[0];
            edge.size = +edge.size[0];
            edge.network_id = +network;
            facts.push(edge);
        }
    }

    dna_attr.facts = facts;
}

/*
 * ==========================================================================
 * ==========================================================================
 */

/**
 *
 * Parses the main json file containing information
 * regarding the nodes and edges of the file.
 */
var run_dna_visualisation = function (network_id) {
    // create a crossfilter dimension
    setup_circle_svg();
    set_up_network_data(dna_attr.facts);

    // Create the dimensions
    create_network_dimension();
    create_network_type_dimension();
    create_interaction_start_dimension();
    create_interaction_end_dimension();

    // Filter on network 0 to start
    filter_on_network(network_id);

    dna_attr.curr_network_id = network_id;

    var nodes = filter_on_network_type('interaction');
    var edges = filter_on_network_type('edge');

    dna_attr.rect.colour_inner = d3.scaleLinear().domain([1, nodes.length])
            .interpolate(d3.interpolateHcl)
            .range([d3.rgb("#007AFF"), d3.rgb('#FFF500')]);

    dna_attr.rect.colour_outer = d3.scaleOrdinal(d3.schemeCategory20b);

    var ordered_nodes = order_nodes(nodes);

    data['ordered_nodes'] = ordered_nodes;

    data.edges = edges;

    draw_interaction_points(ordered_nodes);

    run_fdg({'nodes': ordered_nodes, 'links': edges});
    ;

};


/**
 *
 * Helper that changes the JSON object from being an array
 * to a string or int (i.e. makes the first element of the object
 * the only element.)
 */
var clean_json = function (values) {
    var clean_values = {};
    for (var v in values[0]) {
        var cleaned = {};
        for (var v_inner in values[0][v]) {
            cleaned[v_inner] = values[0][v][v_inner][0];
        }
        clean_values[values[0][v].label] = cleaned;
    }
    return clean_values;
};


/**
 * Sort the nodes descending based on the number of edges.
 */
var sort_nodes = function (nodes) {
    nodes.sort(function (a, b) {
        return a.start - b.start;
    });
    return nodes;
};


/**
 * Orders the nodes by their start position
 */
var order_nodes = function (nodes) {
    //var region_len = full_circle / (Object.keys(nodes).length - 1);
    var pi = Math.PI;
    // Want to calculate the maximum distance between two nodes.
    ordered_nodes = sort_nodes(nodes);
    var num_nodes = ordered_nodes.length;
    var radius = dna_attr.circle.radius;
    var width = 0;
    var angle_size = (1 / (num_nodes / 2)) * pi;
    var line_pts = []; // Create a line going in and out of the points.
    for (var n = 0; n < num_nodes; n++) {
        ordered_nodes[n]['actual_id'] = 'n' + n;
        var this_node = ordered_nodes[n];
        // Store the next node in the dictionary
        ordered_nodes[n]['next'] = ordered_nodes[n + 1];
        /**
         * Below is for drawing the DNA strand
         */
        var start_angle = (n / (num_nodes / 2)) * pi;///(n * region_len);
        var end_angle = (n / (num_nodes / 2)) * pi;//(n * region_len);

        var this_radius = (radius / this_node.num_edges) + dna_attr.circle.min_radius;
        this_node.radius = this_radius;
        this_node.x = (this_radius * Math.cos(start_angle)) + (width / 2);
        this_node.y = (this_radius * Math.sin(start_angle)) + (width / 2);

        if (ordered_nodes[n + 1] != undefined) {
            var node_diff = ordered_nodes[n + 1].start - this_node.end;
            if (node_diff > data.max_dist) {
                data.max_dist = node_diff;
            }
            if (node_diff < data.min_dist) {
                data.min_dist = node_diff;
            }
            this_node['len_to_next'] = ((nodes[n].next.start - nodes[n].start) / data.max_dist);
            // Here we want to add curvy bits for each log point


        } else {
            this_node['len_to_next'] = 0;
        }

        this_node['colour'] = dna_attr.rect.colour_inner(n % 20);
        this_node['stroke-colour'] = dna_attr.rect.colour_outer(this_node.chrom);

        // Save the start and end angle to the node
        ordered_nodes[n].start_angle = start_angle;
        ordered_nodes[n].end_angle = end_angle;

    }

var nodes_to_draw = [];
    for (var n = 0; n < num_nodes; n++) {

        var this_node = ordered_nodes[n];
        var next_node = ordered_nodes[n + 1];
        line_pts.push({'x': this_node.x, 'y': this_node.y});
        if (next_node != undefined) {
            var curves = Math.round(Math.log10(node_diff));
            var prev_node = ordered_nodes[n - 1];

            if (prev_node != undefined) {

                var r1 = this_node.radius;
                var r2 = next_node.radius;
                var mid_rad = (r2 - r1) / curves;
                var a0 = this_node.start_angle - (angle_size / 3);
                var a1 = this_node.start_angle;
                var a2 = this_node.start_angle;
                var a3 = this_node.start_angle + (angle_size / 3);

                var r0 = (this_node.radius * curves) / 1.5;

                // Select the nodes that are between this and the next point
                // i.e. the ones not part of this network.
                remove_network_filters();
                var half_way = (this_node.start + next_node.start) / 2;
                var first = true;
                line_pts.push({'x': r0 * Math.cos(a0), 'y': r0 * Math.sin(a0)});

                var other_interactions = filter_interactions_on_position(this_node.start, next_node.end)
                for (var p in other_interactions) {
                    if (other_interactions[p].network_id != dna_attr.curr_network_id) {
                        // Want to add a line point
                        var lgpt = Math.log10(other_interactions[p].start);
                        var rad = this_node.radius * (Math.log10(other_interactions[p].start));
                        if (other_interactions[p].start < half_way) {
                            nodes_to_draw.push({'x': rad * Math.cos(a0), 'y': rad * Math.sin(a0), 'network': other_interactions[p].network_id})
                            line_pts.push({'x': rad * Math.cos(a0), 'y': rad * Math.sin(a0)});
                        } else {
                            if (first) {

                                line_pts.push({'x': r1 * curves * Math.cos(a2), 'y': r1 * curves * Math.sin(a2)});
                                first = false;
                            }
                           nodes_to_draw.push({'x': rad * Math.cos(a3), 'y': rad * Math.sin(a3), 'network': other_interactions[p].network_id});

                            line_pts.push({'x': rad * Math.cos(a3), 'y': rad * Math.sin(a3)});
                        }
                    }
                }

                if (first) {
                    line_pts.push({'x': r1 * curves * Math.cos(a2), 'y': r1 * curves * Math.sin(a2)});
                    line_pts.push({'x': r0 * Math.cos(a3), 'y': r0 * Math.sin(a3)});
                }


            }
        }
    }
    // Save the relative logs of the distances
    data.log_min_dist = Math.log10(data.min_dist);
    draw_line(line_pts, "black");
    draw_interaction_points(nodes_to_draw, true);
    return ordered_nodes;
}

/**
 * Method for calculating the relative distance.
 *
 * Uses a logarithmic scale based on proportion larger than the smallest distance.
 * Adds 1 to each --> this means that we have at min 1 loop.
 * Each exta extends to another loop.
 */
var calc_relative_loop_dist = function (dist_to_next) {
    var log_dist_to_next = Math.log10(dist_to_next);
    return Math.round(log_dist_to_next - data.log_min_dist + 1);
}


/**
 *
 * Gets the max and min of the position.
 */
var get_max_min = function (nodes) {
    for (var n in nodes) {
        var node = nodes[n];
        if (node.end > data.max_position) {
            data.max_position = node.end;
        }
        if (node.start < data.min_position) {
            data.min_position = node.start;
        }
    }
    // Update the selected network start and end
    dna_attr.selected_network_start = data.min_position;
    dna_attr.selected_network_end = data.max_position;
}


function getPosition(force) {
    return force.nodes().map(function (node) {
        return {
            id: node.id,
            name: node.network_id,
            group: node.group,
            x: node.x,
            y: node.y
        }
    })
}

var run_fdg = function (graph) {
    var margin = {top: 0, right: 0, bottom: 50, left: 10};

    var svg = d3.select("svg#interactionvis"),
            width = +svg.attr("width"),
            height = +svg.attr("height")

    // Remove all existing content
    svg.selectAll("*").remove();

    svg = svg.append("g")
            .attr("transform", "translate(" + margin.left +
                    "," + margin.top + ")");
    dna_attr.fdg_svg = svg;

    var simulation = d3.forceSimulation()
            .force("link", d3.forceLink().id(function (d) {
                return d.id;
            }))
            .force("charge", d3.forceManyBody())
            .force("center", d3.forceCenter(width / 2, height / 2))
            .on('end', function () {
                // Update the thread vis
                // draw_3D_nodes(data['placed_nodes'], threeD_attr);
            });


    var link = svg.append("g")
            .attr("class", "links")
            .selectAll("line")
            .data(graph.links)
            .enter().append("line")
            .attr("stroke", function (d) {
                return "black";//d.colour;
            })
            .attr("opacity", function (d) {
                return 1;//d.value.split("-")[1];
            })
            .attr("stroke-width", 2);

    var node = svg.append("g")
            .attr("class", "nodes")
            .selectAll("circle")
            .data(graph.nodes)
            .enter().append("circle")
            .attr("id", function (d) {
                return "fdg-" + d.id;
            })
            .attr("r", dna_attr.vis.fdg_rad)
            .attr("class", "nodes")
            .attr("fill", function (d) {
                return d.colour;
            })
            .attr("opacity", function (d) {
                return 1;//d.group.split("-")[1];
            })

            .call(d3.drag()
                    .on("start", dragstarted)
                    .on("drag", dragged)
                    .on("end", dragended))
            .on("mouseover", function (d) {

                dna_attr.vis.grp.select("#interaction-" + d.id).attr("fill", "black");
                dna_attr.vis.grp.select("#interaction-" + d.id).attr("opacity", 1);
                dna_attr.vis.grp.select("#interaction-" + d.id).attr("r", 2 * dna_attr.rect.width);
            })
            .on("mouseout", function (d) {
                dna_attr.vis.grp.select("#interaction-" + d.id).attr("fill", d.colour);
                dna_attr.vis.grp.select("#interaction-" + d.id).attr("r", dna_attr.rect.width);
            })

    simulation
            .nodes(graph.nodes)
            .on("tick", ticked);

    simulation.force("link")
            .links(graph.links);

    function ticked() {
        link
                .attr("x1", function (d) {
                    return d.source.x;
                })
                .attr("y1", function (d) {
                    return d.source.y;
                })
                .attr("x2", function (d) {
                    return d.target.x;
                })
                .attr("y2", function (d) {
                    return d.target.y;
                });

        node
                .attr("cx", function (d) {
                    return d.x;
                })
                .attr("cy", function (d) {
                    return d.y;
                });
    }


    function dragstarted(d) {
        if (!d3.event.active)
            simulation.alphaTarget(0.3).restart();
        d.fx = d.x;
        d.fy = d.y;
    }

    function dragged(d) {
        d.fx = d3.event.x;
        d.fy = d3.event.y;
    }

    function dragended(d) {
        if (!d3.event.active)
            simulation.alphaTarget(0);
        d.fx = null;
        d.fy = null;
    }

}

var setup_circle_svg = function () {
    var margin = {top: 400, right: 100, bottom: 50, left: 400};

    var svg = d3.select("svg#strandvis"),
            width = +svg.attr("width") - margin.left - margin.right,
            height = +svg.attr("height") - margin.top - margin.bottom;

    // Remove all existing content
    svg.selectAll("*").remove();

    var grp = svg.append("g")
            .attr("transform", "translate(" + margin.left +
                    "," + margin.top + ")rotate(90)");

    dna_attr.svg = svg;
    dna_attr.vis.grp = grp;
    dna_attr.vis.width = width;
    dna_attr.vis.height = height;
    dna_attr.vis.margin = margin;

}


var draw_interaction_points = function (nodes) {

    dna_attr.vis.grp.selectAll("interaction-pts")
            .data(nodes)
            .enter().append("circle")
            .attr('id', function (d) {
                return "interaction-" + d.id;
            })
            .attr('class', 'interaction')
            .attr("fill", function (d) {
                return d.colour;
            })
            .attr("cx", function (d) {
                return d.x;
            })
            .attr("cy", function (d) {
                return d.y;
            })
            .attr("r", 6)
            .on("mouseover", function (d) {

                dna_attr.fdg_svg.select("#fdg-" + d.id).attr("fill", "black");
                dna_attr.fdg_svg.select("#fdg-" + d.id).attr("opacity", 1);
                dna_attr.fdg_svg.select("#fdg-" + d.id).attr("r", 2 * dna_attr.vis.fdg_rad);
            })
            .on("mouseout", function (d) {
                dna_attr.fdg_svg.selectAll(".nodes").attr("opacity", 1);
                dna_attr.fdg_svg.select("#fdg-" + d.id).attr("fill", d.colour);
                dna_attr.fdg_svg.select("#fdg-" + d.id).attr("r", dna_attr.vis.fdg_rad);
            })
                    .on("click", function(d) {
                        run_dna_visualisation(d.network);
            })
}



/**
 *
 * Simple line function that determins the interpolation.
 */
var line_function = d3.line()
        .x(function (d) {
            return d.x;
        })
        .y(function (d) {
            return d.y;
        })
        .curve(d3.curveNatural)//curveLinear);//curveCardinal);


/**
 *
 * Draw the DNA strand based on the points added above.
 */
var draw_line = function (line_data, colour) {
    dna_attr.vis.grp.append("path")
            .attr("d", line_function(line_data))
            .attr("fill", "none")
            .attr("stroke", colour)
            .attr("stroke-width", dna_attr.dna_strand.stroke_width);

}


/**
 * ============================================================================
 * ============================================================================
 */

var draw_other_interactions = function () {
    // Remove the filter on the network
    remove_network_filters();

    processed_data.network_dimension.filterAll();

    // Draw the first half
    draw_interaction_positions(filter_interactions_on_position(dna_attr.network_range_start,
            dna_attr.selected_network_start), dna_attr.vis.left_grp,
            dna_attr.vis.scale.x_left);

    draw_interaction_positions(filter_interactions_on_position(dna_attr.selected_network_end,
            dna_attr.network_range_end), dna_attr.vis.right_grp,
            dna_attr.vis.scale.x_right);

    // Remove the network filter
    processed_data.network_type_dimension.filterAll();

}
