/**
 * Script for creating a three D visualisatino of the nodes.
 * Modeified a scatterplot implementaion using D3 and three.JS from 
 * http://bl.ocks.org/phil-pedruco/9852362.
 */

//
//function createTextCanvas(text, color, font, size) {
//    size = size || 16;
//    var canvas = document.getElementById('threeDcanvas');
//    var ctx = canvas.getContext('2d');
//    var fontStr = (size + 'px ') + (font || 'Arial');
//    ctx.font = fontStr;
//    var w = ctx.measureText(text).width;
//    var h = Math.ceil(size);
//    canvas.width = 400;
//    canvas.height = 200;
//    context.clearRect(0, 0, canvas.width, canvas.height);
//    ctx.font = fontStr;
//    ctx.fillStyle = color || 'black';
//    ctx.fillText(text, 0, Math.ceil(size * 0.8));
//    return canvas;
//}
//
//function createText2D(text, color, font, size, segW, segH) {
//    var canvas = createTextCanvas(text, color, font, size);
//    var plane = new THREE.PlaneGeometry(canvas.width, canvas.height, segW, segH);
//    var tex = new THREE.Texture(canvas);
//    tex.needsUpdate = true;
//    var planeMat = new THREE.MeshBasicMaterial({
//        map: tex,
//        color: 0xffffff,
//        transparent: true
//    });
//    var mesh = new THREE.Mesh(plane, planeMat);
//    mesh.scale.set(0.5, 0.5, 0.5);
//    mesh.doubleSided = true;
//    return mesh;
//}

// from http://stackoverflow.com/questions/5623838/rgb-to-hex-and-hex-to-rgb
function hexToRgb(hex) { //TODO rewrite with vector output
    var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
    return result ? {
        r: parseInt(result[1], 16),
        g: parseInt(result[2], 16),
        b: parseInt(result[3], 16)
    } : null;
}

var set_up_3D_attr = function () {
    document.getElementById('threeDvis').innerHTML = "";
    var renderer = new THREE.WebGLRenderer({
        antialias: true,
        alpha: true
    });

    var w = 500;
    var h = 400;
    renderer.setSize(w, h);
    renderer.setClearColor(0xffffff, 0);

    document.getElementById('threeDvis').appendChild(renderer.domElement);



// PerspectiveCamera inputs: VIEW_ANGLE, ASPECT, NEAR, FAR
    var camera = new THREE.PerspectiveCamera(45, w / h, 1, 100000);
    camera.position.z = 120;
    camera.position.x = 60;
    camera.position.y = 60;


    var scene = new THREE.Scene();
    //scene.background = new THREE.Color( 0xff0000 );

    var vis = new THREE.Object3D();
    scene.add(vis);

    vis.rotation.y = 0;



    var format = d3.format("+.3f");

    var threeD_attr = {};
    threeD_attr['camera'] = camera;
    threeD_attr['renderer'] = renderer;
    threeD_attr['scene'] = scene;
    threeD_attr['vis'] = vis;
    threeD_attr['format'] = format;
    return threeD_attr;
}

function v(x, y, z) {
    return new THREE.Vector3(x, y, z);
}

var add_vis_to_scene = function(threeD_attr) {
    threeD_attr.vis = new THREE.Object3D();
    threeD_attr.scene.add(threeD_attr.vis);

    threeD_attr.vis.rotation.y = 0;
  
}
var calc_up_or_down = function(up_down_val, scale_factor) {
    var max_val = 20;
    if (up_down_val % 4 == 0) {
        return -max_val * scale_factor;
    } 
    if (up_down_val % 4 == 1) {
        return max_val * scale_factor;
    }
    if (up_down_val % 4 == 2) {
        return max_val * scale_factor;
    }
    return max_val * scale_factor;
};

var calc_line_points = function (threeD_attr, xScale, yScale, zScale) {
    var edges = data['links'];
    var nodes = data['placed_node_dict'];
    for (var e in edges) {
        var edge = edges[e];
        var lineGeo = new THREE.Geometry();
        
        var lineMat = new THREE.LineBasicMaterial({
            color: edge.colour,
            lineWidth: 1,
            opacity: 1
        });

        lineGeo.vertices.push(
                v(-xScale(edge.source.x),
                        -yScale(edge.source.y),
                        zScale(0)),
                
                v(-xScale((edge.source.x + edge.target.x)/2),
                        -yScale((edge.source.y + edge.target.y)/2),
                        zScale(0)),//calc_up_or_down(edge.source.id.substr(1)))),
                        
                v(-xScale(edge.target.x),
                        -yScale(edge.target.y),
                        zScale(0)));

        var line = new THREE.Line(lineGeo, lineMat);

        line.type = THREE.Lines;
        //threeD_attr.vis.add(line);

    }
    
    for (var n = 0; n < Object.keys(nodes).length - 1; n ++) {
        var source = nodes[n];
        var target = nodes[n + 1];
        var lineGeo = new THREE.Geometry();
        var opacity = (n % Object.keys(nodes).length) / 10  + 0.2;
        var lineMat = new THREE.LineBasicMaterial({
            color: 'black',
            lineWidth: 5,
            opacity: opacity
        });

        lineGeo.vertices.push(
                v(-xScale(source.x),
                        -yScale(source.y),
                        zScale(0)),
                
                v(-xScale((source.x + target.x)/2),
                        -yScale((source.y + target.y)/2),
                        zScale(calc_up_or_down(source.id.substr(1), source.len_to_next))),
                        
                v(-xScale(target.x),
                        -yScale(target.y),
                        zScale(0)));

        var line = new THREE.Line(lineGeo, lineMat);

        line.type = THREE.Lines;
        threeD_attr.vis.add(line);

    }    
    
    
}

/**
 * Renders the 3D nodes.
 */
var draw_3D_nodes = function (data, threeD_attr) {
    while (threeD_attr.scene.children.length > 0) {
        threeD_attr.scene.remove(threeD_attr.scene.children[0]);
    }
    add_vis_to_scene(threeD_attr);
    
    var unfiltered = [],
            lowPass = [],
            highPass = [];

    for (var i in data) {
        var d = data[i];
        unfiltered[i] = {
            x: +d.x,
            y: +d.y,
            z: 0, //+d.z
            colour: d.colour
        };
        lowPass[i] = {
            x: +d.x - d.vx, //lp_x,
            y: +d.y - d.vy, //lp_y,
            z: -1, //+d.lp_z
            colour: d.colour
        };
        highPass[i] = {
            x: +d.x + d.vx, //hp_x,
            y: +d.y + d.vy, //hp_y,
            z: 1, //+d.hp_z
            colour: d.colour
        }
    }

    var xExent = d3.extent(unfiltered, function (d) {
        return d.x;
    }),
            yExent = d3.extent(unfiltered, function (d) {
                return d.y;
            }),
            zExent = d3.extent(unfiltered, function (d) {
                return d.z;
            });

    var vpts = {
        xMax: xExent[1],
        xCen: (xExent[1] + xExent[0]) / 2,
        xMin: xExent[0],
        yMax: yExent[1],
        yCen: (yExent[1] + yExent[0]) / 2,
        yMin: yExent[0],
        zMax: 20, //zExent[1],
        zCen: 0, //(zExent[1] + zExent[0]) / 2,
        zMin: -20//zExent[0]
    }

    var xScale = d3.scaleLinear()
            .domain(xExent)
            .range([-50, 50]);
    var yScale = d3.scaleLinear()
            .domain(yExent)
            .range([-50, 50]);
    var zScale = d3.scaleLinear()
            .domain([-20, 20])
            .range([-50, 50]);

calc_line_points(threeD_attr, xScale, yScale, zScale);

    var mat = new THREE.ParticleBasicMaterial({
        vertexColors: true,
        size: 10
    });

    var pointCount = unfiltered.length;

    var pointGeo = new THREE.Geometry();

    for (var i = 0; i < pointCount; i++) {
        var x = -xScale(unfiltered[i].x);
        var y = -yScale(unfiltered[i].y);
        var z = zScale(unfiltered[i].z);
        var geometry = new THREE.SphereGeometry(3, 20, 30, 0, Math.PI * 2, 0, Math.PI * 2);
var material = new THREE.MeshBasicMaterial({ 
    color: unfiltered[i].colour
} );

var cube = new THREE.Mesh(geometry, material);

threeD_attr.vis.add(cube);
cube.position.set(x, y, z);

//        pointGeo.vertices.push(new THREE.Vector3(x, y, z));
//
//        //pointGeo.vertices[i].angle = Math.atan2(z, x);
//        //pointGeo.vertices[i].radius = Math.sqrt(x * x + z * z);
//        //pointGeo.vertices[i].speed = (z / 100) * (x / 100);
//
//        pointGeo.colors.push(new THREE.Color().setRGB(
//                hexToRgb(unfiltered[i].colour).r / 255,
//                hexToRgb(unfiltered[i].colour).g / 255,
//                hexToRgb(unfiltered[i].colour).b / 255
//                ));
    }

//    var points = new THREE.ParticleSystem(pointGeo, mat);
//    threeD_attr.vis.add(points);

    threeD_attr.renderer.render(threeD_attr.scene, threeD_attr.camera);

    var paused = false;
    var last = new Date().getTime();
    var down = false;
    var sx = 0,
            sy = 0;

    window.onmousedown = function (ev) {
        down = true;
        sx = ev.clientX;
        sy = ev.clientY;
    };

    window.onmouseup = function () {
        down = false;
    };

    window.onmousemove = function (ev) {
        if (down) {
            var dx = ev.clientX - sx;
            var dy = ev.clientY - sy;
            threeD_attr.vis.rotation.y += dx * 0.01;
            threeD_attr.camera.position.y += dy;
            sx += dx;
            sy += dy;
        }
    }

    var animating = false;
    window.ondblclick = function () {
        animating = !animating;
    };

    function animate(t) {
        if (!paused) {
            last = t;
            if (animating) {
                var v = pointGeo.vertices;
                for (var i = 0; i < v.length; i++) {
                    var u = v[i];
                    console.log(u)
                    u.angle += u.speed * 0.01;
                    u.x = Math.cos(u.angle) * u.radius;
                    u.z = Math.sin(u.angle) * u.radius;
                }
                pointGeo.__dirtyVertices = true;
            }
            threeD_attr.renderer.clear();
            threeD_attr.camera.lookAt(threeD_attr.scene.position);
            threeD_attr.renderer.render(threeD_attr.scene, threeD_attr.camera);
        }
        window.requestAnimationFrame(animate, threeD_attr.renderer.domElement);
    }
    ;
    animate(new Date().getTime());
    onmessage = function (ev) {
        paused = (ev.data == 'pause');
    }
    ;
}
